import pandas as pd
from pathlib import Path
from dataclasses import dataclass
from .dawa import *
import csv

@dataclass
class Geokoder:
    """En Class, som tager i mod en sti til en csv eller en xlsx fil, og kan geokode den.

    Parameters:
        filepath (str): Placeringen af filen.
    """
    filepath: str = None
    encoding: str = None

    def __post_init__(self):
        if type(self.filepath) == str:
            self.filepath = Path(self.filepath)
            self.extension = self.filepath.suffix

    def progbar(self, curr, total, full_progbar):
        frac = curr / total
        filled_progbar = round(frac * full_progbar)
        print('\r', '#' * filled_progbar + '-' * (full_progbar - filled_progbar), f'[{frac:>7.2%}]', end='')

    def geocode_df(self, dataframe, cols):
        total_len = len(dataframe.index)
        for index, row in dataframe.iterrows():
            search_string = ' '.join([str(row[col]) for col in cols])
            adresse = Adressesoeg(q=search_string)
            adresse_data = adresse.info()
            if len(adresse_data) > 0:
                koordinat = adresse.get_koordinater(adresse_data[0])
                dataframe.at[index, 'x'] = koordinat[0]
                dataframe.at[index, 'y'] = koordinat[1]
            else:
                adresse_vask = Adressevasker(betegnelse=search_string)
                response_vask = adresse_vask.info()
                adr_id = response_vask['resultater'][0]['adresse']['id']
                rigtig_adresse = Adresseopslag(id=adr_id)
                rigtig_adresse_data = rigtig_adresse.info()
                try:
                    koordinat = rigtig_adresse.get_koordinater(rigtig_adresse_data)
                    dataframe.at[index, 'x'] = koordinat[0]
                    dataframe.at[index, 'y'] = koordinat[1]
                except:
                    dataframe.at[index, 'x'] = 'NaN'
                    dataframe.at[index, 'y'] = 'NaN'
            self.progbar(index + 1, total_len, 100)
        return dataframe

    def geokod_file(self, save=False, cols=None):

        if self.filepath == None:
            raise Exception(f'filepath ikke defineret!')
        elif not self.filepath.exists():
            raise FileNotFoundError(f'{self.filepath} findes ikke!')

        if self.extension == '.csv':
            sep = self.find_csv_sep()
            df = pd.read_csv(self.filepath, sep=sep, encoding=self.encoding)
            df_geokod = self.geocode_df(df, cols)
            if save == True:
                df_geokod.to_csv(str(self.filepath.parent.joinpath(f'{self.filepath.stem}_geokodet{self.filepath.suffix}')))
            else:
                return df_geokod

        elif self.extension =='.xlsx':
            df = pd.read_excel(self.filepath)
            df_geokod = self.geocode_df(df, cols)
            if save == True:
                df_geokod.to_excel(str(self.filepath.parent.joinpath(f'{self.filepath.stem}_geokodet{self.filepath.suffix}')))
            else:
                return df_geokod

    def find_csv_sep(self):
        sniffer = csv.Sniffer()
        with open(self.filepath, 'r') as f:
            line = f.readline()
            dialect = sniffer.sniff(line)
        return dialect.delimiter



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydawa']

package_data = \
{'': ['*']}

install_requires = \
['openpyxl>=3.0.3,<4.0.0',
 'pandas>=1.0.3,<2.0.0',
 'requests>=2.23.0,<3.0.0',
 'xlrd>=1.2.0,<2.0.0']

setup_kwargs = {
    'name': 'pydawa',
    'version': '0.5.1',
    'description': 'En API wrapper omkring Danmarks Adressers Web API',
    'long_description': 'PyDawa\n======\nEn simple API wrapper omkring Danmarks Adressers Web API (DAWA).\n\nFormålet med dette modul er at give nem adgang til DAWA i Python. Grunden til, at jeg begyndte at arbejde på dette modul er, at jeg havde skrevet et par scripts i python, der brugte API\'et, men der var ingen "nem" adgang til det. Plus, jeg var nysgerrig over, om jeg kunne skrive et modul og uploade til pypi.\n\nDet her projekt er et "work in progress", så jeg kommer til at tilføje funktionalitet, når jeg har brug for den. Jeg håber, at folk har lyst til at hjælpe med det.\n\nAfhængigheder\n* Requests\n\nInstallation\n----\nInstaller med pip:\n\n```$ pip install pydawa```\n\nBrug\n-----\n\nIndtil videre, består modulet kun af tre classer:\n\n1. Adressesoeg\n2. Adresseopslag\n3. Adressevasker\n\n### Adressesoeg()\nSøg efter en adresse med vejnavn, husnr og postnummer.\n\n```python\nimport pydawa\n\nadresse = pydawa.Adressesoeg(vejnavn=\'Dronning Dagmars vej\', husnr=\'200\', postnr=\'3650\')\nresponse = adresse.info()\n```\n`info()` metoden henter data fra dawa api\'et og returnerer en dictionary med respons.\n\nMan kan også søge med en tekststring.\n```python\nimport pydawa\n\nadresse = pydawa.Adressesoeg(q=\'Dronning Dagmars Vej 200, 3650 Ølstykke\')\nresponse = adresse.info()\n```\n\n### Adresseopslag\nSøg efter en adresse med adressens unikke id.\n\n```python\nimport pydawa\n\nadresse = pydawa.Adresseopslag(id=id)\nresponse = adresse.info()\n```\n`info()` metoden henter data fra dawa api\'et og returnerer en dictionary med respons.\n\n### Adressevasker\nDatavask af adressebetegnelse. Modtager en adressebetegnelse og returnerer en eller flere adresser, der bedst matcher.\n\n```python\nimport pydawa\n\nadresse = pydawa.Adressevasker(betegnelse=adressebetegnelse)\nresponse = adresse.info()\n```\n\n### Koordinater for adresser\nMan kan hente koordinater for en given adresse ved at bruge _get_koordinater_ metoden fra _adresse_ variablen i examplerne ovenfor.\n\n```python\nimport pydawa\n\nadresse = pydawa.Adresseopslag(id=id)\nresponse = adresse.info()\n\nkoordinater = adresse.get_koordinater(response[0])\n```\nDen metode tager et json object. Indtil videre se returner både _Adressesoeg_ og _Adresseopslag_ en liste, så derfor bruger man _response[0]_ som input. \nMan kan kun hente koordinater fra _Adressesoeg_ og _ Adresseopslag_, fordi _Adressevasker_ returnerer ikke koordinater.\n\n### Reverse geokodning\nHvis man har koordinater og gerne vil finde den nærmeste adresse.\n\n```python\nimport pydawa\n\nkoordinater = (12.18, 55.78)\n\nadresse = pydawa.Reverse(koordinater=koordinater)\nresponse = adresse.info()\n```\n\nGeokoder\n=========\nHvis man har en csv eller xlsx fil, som indeholder adresser, som skal geokodes, så kan man bruge den her. Peg på filen og gem resultatet enten som en Pandas dataframe, hvis du skal bearbejde data mere, eller i en fil i samme mappe.\n\n```python\nimport pydawa\n\ngeokoder = pydawa.Geokoder(\'c:/users/brugernavn/adresser.csv\')\n\n# Gem data i en dataframe\ndf = geokoder.geokod_file(cols=[\'adresse\', \'postnr\', \'By\'])\n\n# Gem data i en fil\ngeokoder.geokod_file(save=True, cols=[\'adresse\', \'postnr\', \'By\'])\n\n```\n',
    'author': 'Daníel Örn Árnason',
    'author_email': 'danielarnason85@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/danielarnason/pydawa.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
